package me.mrletsplay.videobase;

import java.util.List;
import java.util.Objects;

import ch.qos.logback.classic.Level;
import me.mrletsplay.mrcore.json.JSONType;
import me.mrletsplay.mrcore.json.converter.JSONConstructor;
import me.mrletsplay.mrcore.json.converter.JSONConvertible;
import me.mrletsplay.mrcore.json.converter.JSONListType;
import me.mrletsplay.mrcore.json.converter.JSONValue;

public class Config implements JSONConvertible {

	public static final List<String> DEFAULT_FILE_TYPES = List.of("mp4", "mpeg", "mkv", "flv", "avi", "webm", "mov", "wmv");

	@JSONValue
	private String libraryPath;

	@JSONValue
	private boolean readOnly;

	@JSONValue
	private String cachePath;

	@JSONValue
	@JSONListType(JSONType.STRING)
	private List<String> includeFileTypes;

	@JSONValue
	private String logLevel;

	@JSONConstructor
	private Config() {}

	public String getLibraryPath() {
		return libraryPath;
	}

	public boolean isReadOnly() {
		return readOnly;
	}

	public String getCachePath() {
		return cachePath;
	}

	public List<String> getIncludeFileTypes() {
		return includeFileTypes == null ? DEFAULT_FILE_TYPES : includeFileTypes;
	}

	public String getLogLevel() {
		return logLevel == null ? Level.INFO.toString() : logLevel;
	}

	public void validate() {
		Objects.requireNonNull(libraryPath, "libraryPath");
	}

	public static Config createDefault() {
		Config config = new Config();
		config.libraryPath = "library";
		config.readOnly = false;
		config.cachePath = "cache";
		config.includeFileTypes = DEFAULT_FILE_TYPES;
		config.logLevel = Level.INFO.toString();
		return config;
	}

}
