package me.mrletsplay.videobase.library;

import java.util.Map;

import me.mrletsplay.mrcore.json.JSONObject;
import me.mrletsplay.mrcore.json.JSONType;
import me.mrletsplay.mrcore.json.converter.JSONConstructor;
import me.mrletsplay.mrcore.json.converter.JSONConvertible;
import me.mrletsplay.simplehttpserver.http.validation.JsonObjectValidator;

public class VideoMetadata implements JSONConvertible {

	public static final String
		FIELD_SERIES = "series",
		FIELD_TITLE = "title",
		FIELD_AUTHOR = "author",
		FIELD_INDEX = "index";

	public static final String
		DEFAULT_SERIES = "Unnamed Series",
		DEFAULT_TITLE = "Unnamed Video",
		DEFAULT_AUTHOR = "Unnamed Author";

	public static final int
		DEFAULT_INDEX = 0;

	public static final VideoMetadata DEFAULT_METADATA = new VideoMetadata(new JSONObject(Map.of(
		FIELD_SERIES, DEFAULT_SERIES,
		FIELD_TITLE, DEFAULT_TITLE,
		FIELD_AUTHOR, DEFAULT_AUTHOR,
		FIELD_INDEX, DEFAULT_INDEX
	)));

	public static final JsonObjectValidator VALIDATOR = new JsonObjectValidator()
		.optional(FIELD_SERIES, JSONType.STRING)
		.optional(FIELD_TITLE, JSONType.STRING)
		.optional(FIELD_AUTHOR, JSONType.STRING)
		.optional(FIELD_INDEX, JSONType.INTEGER);

	private VideoMetadata parent;
	private JSONObject metadata;
	private JSONObject cachedEffectiveMetadata;

	@JSONConstructor
	private VideoMetadata() {}

	public VideoMetadata(JSONObject metadata) {
		this.metadata = metadata == null ? new JSONObject() : metadata;
	}

	public String getSeries() {
		return getEffective().optString(FIELD_SERIES).orElse(null);
	}

	public String getTitle() {
		return getEffective().optString(FIELD_TITLE).orElse(null);
	}

	public String getAuthor() {
		return getEffective().optString(FIELD_AUTHOR).orElse(null);
	}

	public Integer getIndex() {
		return getEffective().optInt(FIELD_INDEX).orElse(null);
	}

	public VideoMetadata getParent() {
		return parent;
	}

	public JSONObject getRaw() {
		return new JSONObject(metadata);
	}

	public JSONObject getEffective() {
		if(cachedEffectiveMetadata != null) return cachedEffectiveMetadata;

		JSONObject meta = new JSONObject(parent == null ? null : parent.getEffective());
		for(String key : metadata.keys()) {
			Object value = metadata.get(key);
			if(value == null) { // Setting to null resets to default value if available, otherwise removes attribute
				value = DEFAULT_METADATA.metadata.opt(key).orElse(null);
				if(value == null) {
					meta.remove(key);
					continue;
				}
			}

			meta.set(key, value);
		}

		return cachedEffectiveMetadata = meta;
	}

	@Override
	public void preDeserialize(JSONObject object) {
		this.metadata = object;
	}

	@Override
	public void preSerialize(JSONObject object) {
		for(String key : metadata.keys()) {
			object.set(key, metadata.get(key));
		}
	}

	@Override
	public String toString() {
		return metadata.toString();
	}

	public static VideoMetadata inherit(VideoMetadata parent, VideoMetadata child) {
		VideoMetadata meta = new VideoMetadata(child == null ? null : child.getRaw());
		meta.parent = parent;
		return meta;
	}

}
